/*
    Biribot.ino
    Release 1.0    2015/July
        
  ToDo;
  
    a) Read_distance: Review
    b) Main program: Design Avoiding strategy    
    c) Add magnetometer and implement feedback 
    
  Libray Functions: Avalaible Actions to be programmed in loop()
      
  Move( FORWARD , V_ACT );
  Move( REVERSE , V_ACT );

  Rotate( RIGHT , V_ACT );
  Rotate( LEFT , V_ACT );

  Turn( LEFT_FW , V_ACT );
  Turn( LEFT_RV , V_ACT );
  Turn( RIGHT_FW , V_ACT );
  Turn( RIGHT_RV , V_ACT );

  LM_Test();
  RM_Test();
  Drivetrain_test();

  PAUSE_1;
  PAUSE_2;
  PAUSE_5;

  FOREVER;
*/


#define LED 13

// The following line must be set to #define DEBUG or #define RELEASE
// DEBUG uses the Serial Monitor to test the HCSR04 sensor
#define DEBUG                   

#define FOREVER for(;;)
#define PAUSE_1 delay(1000)
#define PAUSE_2 delay(2000)
#define PAUSE_5 delay(5000)


#define NO                    0
#define SI                  !NO


// Used in Read__distance() HCSR04 function
#define  CONSTANTE           75.0    // 75.0 va bene instead of the original 58.0 that is horrible. Review function

#define DIST_MIN             10.0
#define DIST_ERR              2.0
#define NUM_DEC               2


// Speeds
#define V_MIN                10
#define V_MAX                80
#define V_TEST               25
#define V_ACT                50


// Directions
#define STOP                  0
#define FORWARD               1
#define REVERSE               2
#define LEFT                  3
#define RIGHT                 4
#define LEFT_FW               5     
#define LEFT_RV               6
#define RIGHT_FW              7
#define RIGHT_RV              8



// Interface with hardware

//  Motors ( Servos)
#define LM_PIN                9    //  Left motor    White, Orange
#define RM_PIN               10    //  Right motor   Yellow


//  HCSR04 Ultrasonic sensor
#define HCSR04_TRIG_PIN      11    //  Trigger       Yellow
#define HCSR04_ECHO_PIN      12    //  Echo          Orange


// Servos center positions ( Should be 90 but what can you expect of a 2 bucks Chinese servo? Center = 90 ?? Hahaha 
#define LM_CENTER            80
#define RM_CENTER            83



/*
    Prototypes
*/
float  Read_distance(void);
void Imprime( float );            //Imprime = Print( distance );


void Engine_start( void );
void Engine_stop( void );


void Move( int , int );
void Turn( int , int );
void Rotate( int , int );
void Neutral( void );
void lm_move( int , int );
void rm_move( int , int );

void LM_Test( void );
void RM_Test( void );
void Drivetrain_test( void );




//   Servos ( drive train servos, left-motor & right-motor )
#include <Servo.h> 
Servo lm,rm;





void setup() 
{ 
  // Ultrasonic HCSR04
  pinMode(HCSR04_TRIG_PIN, OUTPUT);
  pinMode(HCSR04_ECHO_PIN, INPUT);
  digitalWrite(HCSR04_TRIG_PIN, LOW);  // Starts HCSR04 trigger pulse to LOW

  #if defined RELEASE
    Engine_start();
    PAUSE_5;          // Nice to wait for a while b4 moving ...
  #endif
  #if defined DEBUG
    Serial.begin(9600);
  #endif
} 



/*

    main loop

*/
void loop() 
{ 
  static float pos = 0.0;

  #if defined RELEASE
    Drivetrain_test();
    FOREVER;                  //   Stop here
  #endif
  
  #if defined DEBUG
    FOREVER
    {
       pos = Read_distance();
       digitalWrite(LED, HIGH);   
       Imprime( pos);
       PAUSE_1;
       digitalWrite(LED, LOW);    
    }
  #endif
  
} 
/*
    
      End of loop()


   __________________________________________________________________________________________________________________________
*/






/*
    Lee_distancia( void )

    Nota: El disparo del sensor ( TRIG_PIN ) se inicia a estado bajo en setup()
          Al final de la funcion sigue en estado bajo
          No hay que iniciarlo al principo de la funcion    
*/
float Read_distance(void)
{
  unsigned long pulso , retorno;
  int cont;

  // Por si acaso ...
  digitalWrite(HCSR04_ECHO_PIN, LOW);  // Starts HCSR04 echo pin to LOW

  // Trigger pulse in HCSR04 sensor. We need only a 5/10 microsg pulse
  digitalWrite(HCSR04_TRIG_PIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(HCSR04_TRIG_PIN, LOW);
    
  //Read the Echo if exists
  pulso = pulseIn(HCSR04_ECHO_PIN, HIGH,1500);  //1500 shortens the range to 4 cm - 20 cm

  return (float) ( pulso/CONSTANTE);
}




#if defined DEBUG

/*
    Imprime( int distancia )
*/
void Imprime( float distancia )
{
  
  Serial.print( "   Distancia ... ");
  Serial.println( distancia , NUM_DEC );
}

#endif




/*
    Engine_start()
*/
void Engine_start( void )
{
  lm.attach(LM_PIN);    
  rm.attach(RM_PIN);  
  Neutral();
}



/*
    Engine_stop()
*/
void Engine_stop( void )
{
  Neutral();
  lm.detach();    
  rm.detach();  
}



/*
    Move()
    
    Move Biribot moving both motors the same direction
    
      pars:
        where: { FORWARD , REVERSE }
        speed : { 0 , V_MAX  }
*/
void Move( int where , int speed )
{
  switch( where )
  {
    case FORWARD:
      lm_move( FORWARD , speed);
      rm_move( FORWARD , speed);
      break;
    case REVERSE:
      lm_move( REVERSE , speed);
      rm_move( REVERSE , speed);
      break;
    default: 
      return;
  }
}



/*
    Turn()
    
    Turn Biribot moving only one motor FORWARD/REVERSE
    
      pars:
        where: { LEFT_FW , LEFT_RV , RIGHT_FW , RIGHT_RV }     ( FW: FORWARD , RV: REVERSE )
        speed : { 0 , V_MAX  }
*/
void Turn( int where , int speed )
{
  switch( where )
  {
    case LEFT_FW:
      lm_move( FORWARD , speed);
      rm_move( STOP , speed);
      break;
    case LEFT_RV:
      lm_move( REVERSE , speed);
      rm_move( STOP , speed);
      break;

    case RIGHT_FW:
      lm_move( STOP , speed);
      rm_move( FORWARD , speed);
      break;
    case RIGHT_RV:
      lm_move( STOP , speed);
      rm_move( REVERSE , speed);
      break;
    default:
      return;
  }
}



/*
    Rotate()
    
    Rotate Biribot moving both motor opposite directions ( ie lm FORWARD , rm REVERSE )
    
      pars:
        where: { LEFT , RIGHT }
        speed : { 0 , V_MAX  }
*/
void Rotate( int where , int speed )
{
  switch( where )
  {
    case LEFT:
      lm_move( FORWARD , speed);
      rm_move( REVERSE , speed);
      break;
    case RIGHT:
      lm_move( REVERSE , speed);
      rm_move( FORWARD , speed);
      break;
    default:
      return;
  }
}



/*
    Neutral()

    Stops motors moving each to center position    
*/
void Neutral(void)
{
  lm.write(LM_CENTER);
  rm.write(RM_CENTER);
}




/*******************************************************************************************************************/





/*
    lm_move()
    
    Move Left Motor 
    
      pars:
        where: { FORWARD , REVERSE , STOP }
        speed : { 0 , V_MAX  }
*/
void lm_move( int where , int speed )
{
  if( (speed > V_MAX) || (speed < V_MIN) )
    return;

  switch( where )
  {
    case FORWARD:
      lm.write(LM_CENTER+speed); 
      break;
    case REVERSE:
      lm.write(LM_CENTER-speed); 
      break;
    case STOP:
    default:
      lm.write(LM_CENTER);
  }
}



/*
    rm_move()

    Move Right Motor 
    
      pars:
        where: { FORWARD , REVERSE , STOP }
        speed : { 0 , V_MAX  }
*/
void rm_move( int where , int speed )
{
  if( (speed > V_MAX) || (speed < V_MIN) )
    return;

  switch( where )
  {
    case FORWARD:
      rm.write(RM_CENTER-speed);
      break;
    case REVERSE:
      rm.write(RM_CENTER+speed);
      break;
    case STOP:
    default:
      rm.write(RM_CENTER);
  }
}



/*
    Left Motor Test
*/
void LM_Test( void )
{
  lm_move( FORWARD , V_TEST );
  PAUSE_2;
  lm_move( STOP , V_TEST );
  PAUSE_2;
  lm_move( REVERSE , V_TEST );
  PAUSE_2;
  lm_move( STOP , V_TEST );
  PAUSE_2;
}



/*
    Right Motor Test
*/
void RM_Test( void )
{
  rm_move( FORWARD , V_TEST );
  PAUSE_2;
  rm_move( STOP , V_TEST );
  PAUSE_2;
  rm_move( REVERSE , V_TEST );
  PAUSE_2;
  rm_move( STOP , V_TEST );
  PAUSE_2;
}



/*
    Right Motor Test
*/
void Drivetrain_test( void )
{
  Move( FORWARD , V_ACT );
  PAUSE_5;
  Neutral();
  PAUSE_2;
  Move( REVERSE , V_ACT );
  PAUSE_5;
  Neutral();
  PAUSE_2;

  Rotate( RIGHT , V_ACT );
  PAUSE_2;
  Neutral();
  PAUSE_2;

  Rotate( LEFT , V_ACT );
  PAUSE_2;
  Neutral();
  PAUSE_2;

  Turn( LEFT_FW , V_ACT );
  PAUSE_2;
  Neutral();
  PAUSE_2;

  Turn( LEFT_RV , V_ACT );
  PAUSE_2;  
  Neutral();
  PAUSE_2;
  
  Turn( RIGHT_FW , V_ACT );
  PAUSE_2;  
  Neutral();
  PAUSE_2;
    
  Turn( RIGHT_RV , V_ACT );
  PAUSE_2;      
  Neutral();
  PAUSE_2;
}      


